function convertToTime(seconds) {
    let minutes = Math.floor(seconds / 60);
    let remainingSeconds = Math.floor(seconds - (minutes * 60));
    if (minutes < 10) minutes = "0" + minutes;
    if (remainingSeconds < 10) remainingSeconds = "0" + remainingSeconds;
    return minutes + ":" + remainingSeconds;
}

function PanelEdition(data) {
    this.tag = data.tag;
    this.readonlyDiv = undefined;
    this.editDiv = undefined;
    this.canEdit = data.canEdit;
    this.popupPanelWidget = data.popupPanelWidget;
}
// eslint-disable-next-line no-unused-vars
PanelEdition.prototype.appendInputEditorDiv = function (parentDiv) {
}
PanelEdition.prototype.closeEdition = function () {
    this.popupPanelWidget.setCurrentOpenedInput(undefined);
    this.editDiv?.hide();
    this.readonlyDiv?.show();
}
PanelEdition.prototype.openEdition = function () {
    this.popupPanelWidget.ensureLastOpenedEditionClosed();
    if (this.canEdit) {
        this.popupPanelWidget.setCurrentOpenedInput(this);
        this.readonlyDiv.hide();
        if (this.editDiv) {
            this.editDiv.show().focus();
        }
    }
}

function AnnotationNamePanelEdition(data) {
    PanelEdition.call(this, data);
}
AnnotationNamePanelEdition.prototype = Object.create(PanelEdition.prototype);
AnnotationNamePanelEdition.prototype.appendInputEditorDiv = function (parentDiv) {
    let that = this;
    this.readonlyDiv = $('<div>');
    this.readonlyDiv
        .attr('id', 'popupNom')
        .addClass('popupRightItem');

    if (this.canEdit) {
        this.readonlyDiv
            .click(function (event) {
                event.stopPropagation();
                that.openEdition.call(that);
            });
    }

    if (this.tag.getMetadata("Rekall->Name")?.length) {
        this.readonlyDiv.html(this.tag.getMetadata("Rekall->Name"));
        this.readonlyDiv.css('color', this.tag.color)
    } else {
        if (this.canEdit) {
            this.readonlyDiv.html('+ Add a name').addClass('empty');
        }
    }

    parentDiv.append(this.readonlyDiv);

    if (this.canEdit) {
        this.editDiv = $('<input>');
        this.editDiv
            .attr('id', 'popupNomInput')
            .attr('type', 'text')
            .attr('value', this.tag.getMetadata("Rekall->Name") ?? '')
            .click(function (event) {
                event.stopPropagation();
            })
            .keyup(function (event) {
                event.stopPropagation();
                if (13 === event.which) {
                    that.closeEdition();
                }
            })
            .hide();
        this.editDiv.val(this.tag.getMetadata("Rekall->Name"));
        parentDiv.append(this.editDiv);
    }
}
AnnotationNamePanelEdition.prototype.closeEdition = function () {
    if (this.canEdit) {
        let newValue = this.editDiv.val();
        this.editDiv.val(newValue);
        if (this.tag.getMetadata('Rekall->Name') !== newValue) {
            this.tag.setMetadata('Rekall->Name', newValue);
            window.app.rekall.Rekall('pubSub').publish('tag.metadata.updated', {
                metadata: "Rekall->Name",
                tag: this.tag
            });
        }

        if (newValue.length) {
            $('#popupNom').html(newValue).removeClass("empty");
        } else {
            $('#popupNom').html('+ Add a name').addClass("empty");
        }
    }
    PanelEdition.prototype.closeEdition.call(this);
}

function AnnotationAuthorPanelEdition(data) {
    PanelEdition.call(this, data);
}
AnnotationAuthorPanelEdition.prototype = Object.create(PanelEdition.prototype);
AnnotationAuthorPanelEdition.prototype.appendInputEditorDiv = function (parentDiv) {
    let that = this;
    this.readonlyDiv = $('<div>');
    this.readonlyDiv.addClass('popupRightItem')
        .attr('id', 'popupAuthor');

    if (this.canEdit) {
        this.readonlyDiv
            .click(function (event) {
                event.stopPropagation();
                that.openEdition.call(that);
            });
    }

    let popupAuthorName = this.tag.getMetadata("Rekall->Author");
    if (popupAuthorName?.length) {
        this.readonlyDiv.html(popupAuthorName);
    } else {
        if (this.canEdit) {
            this.readonlyDiv.html('+ Add an author').addClass('empty');
        }
    }
    parentDiv.append(this.readonlyDiv);


    if (this.canEdit) {
        this.editDiv = $('<input>');
        this.editDiv
            .attr('id', 'popupAuthorInput')
            .attr('type', 'text')
            .attr('value', popupAuthorName ?? '')
            .click(function (event) {
                event.stopPropagation();
            })
            .keyup(function (event) {
                event.stopPropagation();
                if (13 === event.which) {
                    that.closeEdition();
                }
            })
            .addClass('popupInput')
            .hide();
        parentDiv.append(this.editDiv);
    }
}
AnnotationAuthorPanelEdition.prototype.closeEdition = function () {
    if (this.canEdit) {
        let newValue = this.editDiv.val();
        this.editDiv.val(newValue);
        if (this.tag.getMetadata('Rekall->Author') !== newValue) {
            this.tag.setMetadata('Rekall->Author', newValue);
            window.app.rekall.Rekall('pubSub').publish('tag.metadata.updated', {
                metadata: "Rekall->Author",
                tag: this.tag
            });
        }

        if (newValue.length) {
            this.readonlyDiv.html(newValue).removeClass("empty");
        } else {
            this.readonlyDiv.html('+ Add an author').addClass("empty");
        }
    }
    PanelEdition.prototype.closeEdition.call(this);
}

function AnnotationTCPanelEdition(data) {
    PanelEdition.call(this, data);
}
AnnotationTCPanelEdition.prototype = Object.create(PanelEdition.prototype);
AnnotationTCPanelEdition.prototype.appendInputEditorDiv = function (parentDiv) {

    // eslint-disable-next-line no-unused-vars
    function onlyNumber(event) {
        // remove characters that are not number
        $(this).val($(this).val().replace(/\D/g, ''));
    }
    let that = this;

    this.readonlyDiv = $('<div>');
    this.readonlyDiv
        .attr('id', 'popupTC')
        .addClass('popupRightItem')
        .click(function (event) {
            event.stopPropagation();
            that.openEdition.call(that);
        })
        .css("background", that.tag.color);

    this.popupTcInDiv = $('<div>');
    this.popupTcInDiv
        .attr('id', 'popupTCin')
        .addClass('popupTCdisplay')
        .html(convertToTime(that.tag.getTimeStart()))
    this.readonlyDiv.append(this.popupTcInDiv);

    this.popupTcOutDiv = $('<div>');
    this.popupTcOutDiv
        .attr('id', 'popupTCout')
        .addClass('popupTCdisplay')
        .html(convertToTime(that.tag.getTimeEnd()))
    this.readonlyDiv.append(this.popupTcOutDiv);
    parentDiv.append(this.readonlyDiv);

    if (this.canEdit) {
        this.editDiv = $('<div>');
        this.editDiv
            .attr('id', 'popupTCedit')
            .hide()
        this.editDiv.click(function (event) {
            event.stopPropagation();
        })
        this.editDiv.append($('<span>').addClass('popupTClabel').html('start'));
        this.popupTCeditDivMinStart = $('<input class="popupTCeditfield" id="popupTCinMin" maxLength="2" type="text" value="">');
        this.popupTCeditDivMinStart
            .val(Math.floor(that.tag.getTimeStart()/60).toString(10).padStart(2, '0'))
            .click(function (event) {
                event.stopPropagation();
            })
            .on('input', onlyNumber);
        this.editDiv.append(this.popupTCeditDivMinStart);
        this.editDiv.append(':');
        this.popupTCeditDivSecStart = $('<input class="popupTCeditfield" id="popupTCinSec" maxLength="2" type="text" value="">');
        this.popupTCeditDivSecStart
            .val((that.tag.getTimeStart()%60).toString(10).padStart(2, '0'))
            .click(function (event) {
                event.stopPropagation();
            })
            .on('input', onlyNumber);
        this.editDiv.append(this.popupTCeditDivSecStart);
        let popupTCeditStartDivNow = $('<div class="nowTCbtn" id="nowTCin">now</div>');
        popupTCeditStartDivNow.click(function (event) {
            event.stopPropagation();
            let timeCurrent = convertToTime(Math.round(window.app.rekall.Rekall().timeline.timeCurrent));
            that.popupTCeditDivMinStart.val(timeCurrent.split(":")[0]);
            that.popupTCeditDivSecStart.val(timeCurrent.split(":")[1]);
        });
        this.editDiv.append(popupTCeditStartDivNow)
        this.editDiv.append($('<br>'));

        this.editDiv.append($('<span>').addClass('popupTClabel').html('end'));
        this.popupTCeditDivMinEnd = $('<input>');
        this.popupTCeditDivMinEnd
            .attr('id', 'popupTCoutMin')
            .attr('maxlength', '2')
            .attr('type', 'text')
            .addClass('popupTCeditfield')
            .val(Math.floor(that.tag.getTimeEnd()/60).toString(10).padStart(2, '0'))
            .on('input', onlyNumber);

        this.popupTCeditDivMinEnd.click(function (event) {
            event.stopPropagation();
        });
        this.editDiv.append(this.popupTCeditDivMinEnd);
        this.editDiv.append(':');
        this.popupTCeditDivSecEnd = $('<input>');
        this.popupTCeditDivSecEnd
            .attr('id', 'popupTCoutSec')
            .attr('maxlength', '2')
            .attr('type', 'text')
            .addClass('popupTCeditfield')
            .val(Math.floor(that.tag.getTimeEnd()%60).toString(10).padStart(2, '0'))
            .click(function (event) {
                event.stopPropagation();
            })
            .on('input', onlyNumber);
        this.editDiv.append(this.popupTCeditDivSecEnd);
        let popupTCeditEndDivNow = $('<div class="nowTCbtn" id="nowTCin">now</div>');
        popupTCeditEndDivNow.click(function (event) {
            event.stopPropagation();
            let timeCurrent = convertToTime(Math.round(window.app.rekall.Rekall().timeline.timeCurrent));
            that.popupTCeditDivMinEnd.val(timeCurrent.split(":")[0]);
            that.popupTCeditDivSecEnd.val(timeCurrent.split(":")[1]);
        });
        this.editDiv.append(popupTCeditEndDivNow);
        this.editDiv.append('/');
        let popupTcEditEndDivEndVideo = $('<div class="nowTCbtn" id="eovTCout">end of video</div>');
        popupTcEditEndDivEndVideo.click(function (event) {
            event.stopPropagation();
            let endVideo = window.app.rekall.Rekall().videoPlayer.duration();
            let timeEnd = convertToTime(Math.round(endVideo));
            that.popupTCeditDivMinEnd.val(timeEnd.split(":")[0]);
            that.popupTCeditDivSecEnd.val(timeEnd.split(":")[1]);
        });
        this.editDiv.append(popupTcEditEndDivEndVideo);
        parentDiv.append(this.editDiv);
    }
}
AnnotationTCPanelEdition.prototype.closeEdition = function () {
    if (this.canEdit) {
        let isNaN = (maybeNaN) => maybeNaN !== maybeNaN;
        let ensureNotNaN = (text, defaultValue = 0) => {
            let tempVal = parseInt(text);
            if (isNaN(tempVal)) {
                return defaultValue;
            }
            return tempVal;
        }
        let tcInMin = ensureNotNaN(this.popupTCeditDivMinStart.val());
        let tcInSec = ensureNotNaN(this.popupTCeditDivSecStart.val());
        let tcOutMin = ensureNotNaN(this.popupTCeditDivMinEnd.val());
        let tcOutSec = ensureNotNaN(this.popupTCeditDivSecEnd.val());
        let tcIn = (60 * tcInMin) + tcInSec;
        let tcOut = (60 * tcOutMin) + tcOutSec;

        let endVideo = Math.ceil(window.app.rekall.Rekall().videoPlayer.duration());

        if ((tcInMin >= 120) || (tcInSec >= 60) || (tcOutMin >= 120) || (tcOutSec >= 60) || (tcInMin < 0) || (tcInSec < 0) || (tcOutMin < 0) || (tcOutSec < 0)) {
            window.app.rekall.Rekall('pubSub').publish('alert', {message: 'Invalid time code', buttons: "ok"});
        } else if (tcIn > tcOut) {
            window.app.rekall.Rekall('pubSub').publish('alert', {
                message: "Start time must be set before end time",
                buttons: "ok"
            });
        } else if (tcOut > endVideo) {
            window.app.rekall.Rekall('pubSub').publish('alert', {
                message: "End time must not be set after " + convertToTime(endVideo) + " (end of the video)",
                buttons: "ok"
            });
        } else {
            this.tag.setTimeStart(tcIn);
            this.tag.setTimeEnd(tcOut);
            this.popupTcInDiv.html(tcInMin.toString().padStart(2, '0') + ":" + tcInSec.toString().padStart(2, '0'));
            this.popupTcOutDiv.html(tcOutMin.toString().padStart(2, '0') + ":" + tcOutSec.toString().padStart(2, '0'));
            this.popupTCeditDivMinStart.val(tcInMin.toString().padStart(2, '0'));
            this.popupTCeditDivSecStart.val(tcInSec.toString().padStart(2, '0'));
            this.popupTCeditDivMinEnd.val(tcOutMin.toString().padStart(2, '0'));
            this.popupTCeditDivSecEnd.val(tcOutSec.toString().padStart(2, '0'));
            window.app.rekall.Rekall('pubSub').publish('tag.tc.updated', {tag: this.tag});
        }
    }
    PanelEdition.prototype.closeEdition.call(this);
}

function AnnotationCommentPanelEdition(data) {
    PanelEdition.call(this, data);
}
AnnotationCommentPanelEdition.prototype = Object.create(PanelEdition.prototype);
AnnotationCommentPanelEdition.prototype.appendInputEditorDiv = function (parentDiv) {
    let that = this;
    this.readonlyDiv = $('<div>');
    this.readonlyDiv.addClass('popupRightItem')
        .attr('id', 'popupLegende')
        .attr('title', 'Comment');

    if (this.canEdit) {
        this.readonlyDiv
            .click(function (event) {
                event.stopPropagation();
                that.openEdition.call(that);
            });
    }

    let comments = this.tag.getMetadata("Rekall->Comments");
    if (comments && comments.length) {
        this.readonlyDiv.html(comments).removeClass('empty');
    } else {
        if (this.canEdit) {
            this.readonlyDiv.html('+ Add a comment').addClass('empty');
        }
    }
    parentDiv.append(this.readonlyDiv);

    if (this.canEdit) {
        this.editDiv = $('<textarea>');
        this.editDiv
            .attr('id', 'popupLegendeInput')
            .attr('type', 'text')
            .addClass('popupInput')
            .click(function (event) {
                event.stopPropagation();
            });
        this.editDiv.val(comments ? comments : '');
        parentDiv.append(this.editDiv);
    }
}
AnnotationCommentPanelEdition.prototype.closeEdition = function () {
    if (this.canEdit) {
        let newValue = this.editDiv.val();
        this.editDiv.val(newValue);
        if (this.tag.getMetadata('Rekall->Comments') !== newValue) {
            newValue = newValue.replace(/\n/gi, "<br/>");
            this.tag.setMetadata('Rekall->Comments', newValue);
            window.app.rekall.Rekall('pubSub').publish('tag.metadata.updated', {
                metadata: "Rekall->Comments",
                tag: this.tag
            });
        }

        if (newValue.length) {
            this.readonlyDiv.html(newValue).removeClass("empty");
        } else {
            this.readonlyDiv.html('+ Add an author').addClass("empty");
        }
    }
    PanelEdition.prototype.closeEdition.call(this);
}

function AnnotationLinkPanelEdition(data) {
    PanelEdition.call(this, data);
}
AnnotationLinkPanelEdition.prototype = Object.create(PanelEdition.prototype);
AnnotationLinkPanelEdition.prototype.appendInputEditorDiv = function (parentDiv) {
    let that = this;
    this.readonlyDiv = $('<div>');
    this.readonlyDiv.addClass('popupRightItem')
        .attr('id', 'popupLink')
        .attr('title', 'Link');

    if (this.canEdit) {
        this.readonlyDiv
            .click(function (event) {
                event.stopPropagation();
                that.openEdition.call(that);
            });
    }

    let linkValue = this.tag.getMetadata("Rekall->Link");
    if (linkValue && linkValue.length) {
        if (this.canEdit) {
            this.readonlyDiv.html(linkValue).removeClass('empty');
        } else {
            this.readonlyDiv.html("<a href='" + linkValue + "' target='_blank'>" + linkValue + "</a>").removeClass('empty');
        }
    } else {
        if (this.canEdit) {
            this.readonlyDiv.html("+ Add a link").addClass("empty");
        }
    }
    parentDiv.append(this.readonlyDiv);

    if (this.canEdit) {
        this.editDiv = $('<textarea>');
        this.editDiv
            .attr('id', 'popupLinkInput')
            .attr('type', 'text')
            .addClass('popupInput')
            .click(function (event) {
                event.stopPropagation();
            })
            .keyup(function (event) {
                event.stopPropagation();
                if (13 === event.which) {
                    that.closeEdition();
                }
            })
            .val(linkValue)
            .hide();

        parentDiv.append(this.editDiv);
    }
}
AnnotationLinkPanelEdition.prototype.closeEdition = function () {
    if (this.canEdit) {
        let newValue = this.editDiv.val();
        newValue = newValue.trim();
        if (newValue && newValue.length && 0 !== newValue.indexOf("http")) {
            newValue = "http://" + newValue;
        }
        this.editDiv.val(newValue);

        if (this.tag.getMetadata('Rekall->Link') !== newValue) {
            this.tag.setMetadata('Rekall->Link', newValue);
            window.app.rekall.Rekall('pubSub').publish('tag.metadata.updated', {
                metadata: "Rekall->Link",
                tag: this.tag
            });
        }

        if (newValue.length) {
            this.readonlyDiv.html(newValue).removeClass("empty");
        } else {
            this.readonlyDiv.html('+ Add a Link').addClass("empty");
        }
    }
    PanelEdition.prototype.closeEdition.call(this);
}

function AnnotationSpeedPanelEdition(data) {
    PanelEdition.call(this, data);
}
AnnotationSpeedPanelEdition.prototype = Object.create(PanelEdition.prototype);
AnnotationSpeedPanelEdition.prototype.appendInputEditorDiv = function (parentDiv) {
    let that = this;

    let speed = this.tag.getMetadata("Rekall->Speed");
    if (!speed || !speed.trim().length)
        speed = 1;
    if (!this.canEdit && (1 === speed || "1" === speed)){
        return;
    }

    this.readonlyDiv = $('<form>');
    this.readonlyDiv.addClass('popupRightItem')
        .attr('id', 'popupSpeedInput')
        .addClass('flex-row playback_form popupInput popupInputNoHide');

    let input = $('<input>');
    input
        .attr('id', 'annotation_playback_speed_025')
        .attr('type', 'radio')
        .attr('name', 'playback_rate')
        .attr('disabled', this.canEdit ? null : 'disabled')
        .val(0.25);
    let inputLabel = $('<label>');
    inputLabel
        .attr('for', 'annotation_playback_speed_025')
        .html('0.25')
        .addClass('left_menu_item');
    this.readonlyDiv.append(input);
    this.readonlyDiv.append(inputLabel);

    input = $('<input>');
    input
        .attr('id', 'annotation_playback_speed_05')
        .attr('type', 'radio')
        .attr('name', 'playback_rate')
        .prop('disabled', this.canEdit ? null : 'disabled')
        .val(0.5);
    inputLabel = $('<label>');
    inputLabel
        .attr('for', 'annotation_playback_speed_05')
        .html('0.5')
        .addClass('left_menu_item');
    this.readonlyDiv.append(input);
    this.readonlyDiv.append(inputLabel);

    input = $('<input>');
    input
        .attr('id', 'annotation_playback_speed_075')
        .attr('type', 'radio')
        .attr('name', 'playback_rate')
        .attr('disabled', this.canEdit ? null : 'disabled')
        .val(0.75);
    inputLabel = $('<label>');
    inputLabel
        .attr('for', 'annotation_playback_speed_075')
        .html('0.75')
        .addClass('left_menu_item');
    this.readonlyDiv.append(input);
    this.readonlyDiv.append(inputLabel);

    input = $('<input>');
    input
        .attr('id', 'annotation_playback_speed_1')
        .attr('type', 'radio')
        .attr('name', 'playback_rate')
        .attr('disabled', this.canEdit ? null : 'disabled')
        .val(1);
    inputLabel = $('<label>');
    inputLabel
        .attr('for', 'annotation_playback_speed_1')
        .html('normal')
        .addClass('left_menu_item');
    this.readonlyDiv.append(input);
    this.readonlyDiv.append(inputLabel);

    input = $('<input>');
    input
        .attr('id', 'annotation_playback_speed_125')
        .attr('type', 'radio')
        .attr('name', 'playback_rate')
        .attr('disabled', this.canEdit ? null : 'disabled')
        .val(1.25);
    inputLabel = $('<label>');
    inputLabel
        .attr('for', 'annotation_playback_speed_125')
        .html('1.25')
        .addClass('left_menu_item');
    this.readonlyDiv.append(input);
    this.readonlyDiv.append(inputLabel);

    input = $('<input>');
    input
        .attr('id', 'annotation_playback_speed_15')
        .attr('type', 'radio')
        .attr('name', 'playback_rate')
        .attr('disabled', this.canEdit ? null : 'disabled')
        .val(1.5);
    inputLabel = $('<label>');
    inputLabel
        .attr('for', 'annotation_playback_speed_15')
        .html('1.5')
        .addClass('left_menu_item');
    this.readonlyDiv.append(input);
    this.readonlyDiv.append(inputLabel);

    input = $('<input>');
    input
        .attr('id', 'annotation_playback_speed_175')
        .attr('type', 'radio')
        .attr('name', 'playback_rate')
        .attr('disabled', this.canEdit ? null : 'disabled')
        .val(1.75);
    inputLabel = $('<label>');
    inputLabel
        .attr('for', 'annotation_playback_speed_175')
        .html('1.75')
        .addClass('left_menu_item');
    this.readonlyDiv.append(input);
    this.readonlyDiv.append(inputLabel);

    input = $('<input>');
    input
        .attr('id', 'annotation_playback_speed_2')
        .attr('type', 'radio')
        .attr('name', 'playback_rate')
        .attr('disabled', this.canEdit ? null : 'disabled')
        .val(2);
    inputLabel = $('<label>');
    inputLabel
        .attr('for', 'annotation_playback_speed_2')
        .html('2')
        .addClass('left_menu_item');
    this.readonlyDiv.append(input);
    this.readonlyDiv.append(inputLabel);


    this.readonlyDiv.children('input[value="' + speed + '"]').attr('checked', true);
    this.readonlyDiv.children('input[name=playback_rate]').change(function (event) {
        event.stopPropagation();
        that.closeEdition.call(that);
    })

    var title = $('<div>').addClass('popupNewTitles').html('+ Playback rate');
    parentDiv.append(title);
    parentDiv.append(this.readonlyDiv);
}
AnnotationSpeedPanelEdition.prototype.closeEdition = function () {
    if (this.canEdit) {
        let newValue = this.readonlyDiv.children('input:checked').val();
        if (this.tag.getMetadata('Rekall->Speed') !== newValue) {
            this.tag.setMetadata('Rekall->Speed', newValue);
            window.app.rekall.Rekall('pubSub').publish('tag.metadata.updated', {
                metadata: "Rekall->Speed",
                tag: this.tag
            });
        }
    }
}
AnnotationSpeedPanelEdition.prototype.openEdition = function () {
    this.popupPanelWidget.ensureLastOpenedEditionClosed();
    if (this.canEdit) {
        this.popupPanelWidget.setCurrentOpenedInput(this);
    }
}

function AnnotationTagPanelEdition(data) {
    PanelEdition.call(this, data);
    this.annotationInputTemplate = $('<input class="" type="checkbox" id="" value="">')
    this.annotationLabelTemplate = $('<label class="" for=""></label>')
}
AnnotationTagPanelEdition.prototype = Object.create(PanelEdition.prototype);
AnnotationTagPanelEdition.prototype.appendInputEditorDiv = function (parentDiv) {
    let that = this;

    let _labels = getProjectLabels(this.tag);
    let tagLabels = that.tag.getLabels();

    if (!this.canEdit && (!tagLabels || tagLabels.length === 0)){
        return;
    }
    function getProjectLabels(tag) {
        return tag.document.getLabels();
    }

    function appendTagInputWithLabel(panel, currentLabel, panelContainsLabel) {
        let input = panel.annotationInputTemplate.clone();
        let id = 'annotation_label_' + panel.tag.document.project.labels.indexOf(currentLabel);
        input
            .attr('id', id)
            .attr('value', currentLabel)
            .addClass('annotation_labels')
            .prop("checked", panelContainsLabel)
            .hide();

        if(panel.canEdit){
            input
            .change(function (event) {
                event.stopPropagation();
                that.closeEdition.call(that);
                that.openEdition.call(that);
            })
        } else {
            input
                .change(function (event) {
                    event.stopPropagation();
                })
                .attr('disabled', 'disabled');
        }

        let html_label = panel.annotationLabelTemplate.clone();
        html_label
            .attr('for', id)
            .html(currentLabel)
            .addClass('annotation_labels')
            .css('color', panel.tag.color)
            .css('border', '1px solid ' +panel.tag.color);

        panel.readonlyDiv.append(input);
        panel.readonlyDiv.append(html_label);
    }

    this.tagPanelLabel = $('<div class="popupNewTitles">+ Tags</div>');
    parentDiv.append(this.tagPanelLabel);

    if(that.canEdit){
        this.tagCreationDiv = $('<input type="text" id="new_annotation_label" placeholder="Add a new tag" pattern="[a-zA-Zéàè0-9 ]+" title="Only alphanum characters" />');
        this.tagCreationDiv.keypress(function (event) {
            that.openEdition.call(that);
            let keycode = (event.keyCode ? event.keyCode : event.which);
            if (13 !== keycode) {
                return;
            }

            if ($('#new_annotation_label')[0].validity.patternMismatch) {
              that.tagCreationDiv.css('border', '2px solid red');
              return;
            }

            that.tagCreationDiv.css('border', 'initial');
            let labelToAdd = $(this).val();
            let labels = getProjectLabels(that.tag);
            if (labels.indexOf(labelToAdd) === -1) {
                that.tag.document.project.labels.push(labelToAdd);
                appendTagInputWithLabel(that, labelToAdd);
            }
            that.readonlyDiv.children("input[value='" + labelToAdd + '\']').attr('checked', 'checked');
            $(this).val('');
            that.closeEdition.call(that);
        });
        parentDiv.append(this.tagCreationDiv);
    }

    this.readonlyDiv = $('<div>');
    this.readonlyDiv
        .addClass('flex-col popupInput popupInputNoHide')
        .attr('id', 'popupLabelsInput');


    for (let currentLabel of _labels) {
        if (!this.canEdit && !tagLabels.includes(currentLabel))
            continue;
        appendTagInputWithLabel(this, currentLabel, tagLabels.includes(currentLabel));
    }
    parentDiv.append(this.readonlyDiv);
}
AnnotationTagPanelEdition.prototype.save = function () {
    if (this.canEdit) {
        let newValue = '';
        this.readonlyDiv
            .children('input:checked')
            .each(function () {
                newValue += $(this).val() + ';'
            });
        if (newValue.length) {
            newValue = newValue.slice(0, newValue.length - 1);
        }

        if (this.tag.getMetadata('Rekall->Labels') !== newValue) {
            this.tag.setMetadata('Rekall->Labels', newValue);
            window.app.rekall.Rekall('pubSub').publish('tag.metadata.updated', {
                metadata: "Rekall->Labels",
                tag: this.tag
            });
        }
    }
}
AnnotationTagPanelEdition.prototype.closeEdition = function () {
    if (this !== this.popupPanelWidget.getCurrentOpenedInput())
        this.popupPanelWidget.ensureLastOpenedEditionClosed();
    this.save();
}
AnnotationTagPanelEdition.prototype.openEdition = function () {
    if (this !== this.popupPanelWidget.getCurrentOpenedInput()) {
        this.popupPanelWidget.ensureLastOpenedEditionClosed();
        if (this.canEdit) {
            this.popupPanelWidget.setCurrentOpenedInput(this);
        }
    }
}


function PopupPanelWidget(data) {
    this.canEdit = data?.canEdit ?? function () {
        return false;
    };
    this.currentOpenedInput = undefined;
    this.videoWasPlayingOnContentCreation = false;
}
PopupPanelWidget.prototype.getCurrentOpenedInput = function () {
    return this.currentOpenedInput;
}
PopupPanelWidget.prototype.setCurrentOpenedInput = function (input) {
    this.currentOpenedInput = input;
}
PopupPanelWidget.prototype.ensureLastOpenedEditionClosed = function () {
    if (this.currentOpenedInput) {
        this.currentOpenedInput.closeEdition();
        this.currentOpenedInput = undefined;
    }
}
PopupPanelWidget.prototype.createPopupContent = function (tag) {

    let that = this;
    this.videoWasPlayingOnContentCreation = !window.app.rekall.Rekall().timeline.isPaused();
    if (this.videoWasPlayingOnContentCreation) {
        window.app.rekall.Rekall().timeline.pause();
    }

    function createLeftPopup(popupPanelWidget) {
        let that = this;
        let popupLeftDiv = $('#popupLeft');
        let bgColorLeft = this.getTagGradientColor();
        popupLeftDiv
            .empty()
            .css("background", bgColorLeft)
            .attr('keydoc', this.document.key)

        let highlightDiv = $('<div>')
            .attr('id', 'popupSetHighlight')
            .text("★ Highlight")
            .attr("ishighlight", this.getMetadata("Rekall->Highlight")?.length ? "true" : "false")
            .click(function (event) {
                event.stopPropagation();
                let isHL = !!that.getMetadata("Rekall->Highlight");

                if (isHL) {
                    that.setMetadata("Rekall->Highlight", "");
                    $(this).attr("isHighlight", "false").removeClass("selected");
                    $("#popupEdit").removeClass("highlightPopup");
                } else {
                    that.setMetadata("Rekall->Highlight", "true");
                    $(this).attr("isHighlight", "true").addClass("selected");
                    $("#popupEdit").addClass("highlightPopup");
                }

                window.app.rekall.Rekall('pubSub').publish('tag.metadata.updated', {
                    metadata: "Rekall->Highlight",
                    tag: that
                });
            });

        if (this.getMetadata("Rekall->Highlight")?.length) {
            highlightDiv
                .addClass('selected');
        }

        popupLeftDiv.append(highlightDiv);

        let imageBoxDiv = $('<div>');
        imageBoxDiv.addClass('popupLeftItem')
            .attr('id', 'popupImgBox');

        let image = $('<img src="" alt="">');
        let fileDirectLink = this.getDownloadLink();
        if (this.isMarker()) {
            image.attr('src', "../shared/css/images/img-note.png");
        } else {
            if (this.thumbnail.url) {
                image.attr("src", fileDirectLink || this.thumbnail.url);
            } else {
                let type = this.getMetadata("Rekall->Type");
                if (type.indexOf("image") > -1) image.attr("src", "../shared/css/images/img-image.png")
                else if (type.indexOf("pdf") > -1) image.attr("src", "../shared/css/images/img-pdf.png")
                else if (type.indexOf("audio") > -1) image.attr("src", "../shared/css/images/img-music.png")
                else if (type.indexOf("vcard") > -1) image.attr("src", "../shared/css/images/img-user.png")
                else if (type.indexOf("video") > -1) image.attr("src", "../shared/css/images/img-video.png")
                else if (type.indexOf("msword") > -1) image.attr("src", "../shared/css/images/img-word.png")
                else if (type.indexOf("link") > -1) image.attr("src", "../shared/css/images/img-link.png")
                else image.attr("src", "../shared/css/images/img-document.png")
            }

            image
                .attr('id', 'popupImg')
                .click(function (event) {
                    event.stopPropagation();
                    if (that.isLink()) {
                        if (that.getMetadata("Rekall->Link")) window.open(that.getMetadata("Rekall->Link"), '_blank')
                    } else {
                        window.open(fileDirectLink, '_blank');
                    }
                });
        }

        imageBoxDiv.append(image);

        let popupTypeDiv = $('<div>');
        popupTypeDiv
            .attr('id', 'popupType')
            .addClass('popupLeftItem')
            .html(this.getMetadata("Rekall->Type"))
            .css('color', this.color);
        imageBoxDiv.append(popupTypeDiv);

        if (this.getMetadata("Rekall->Type")?.length
            && this.getMetadata("Rekall->Type").split('/')[0] === 'image') {
            if (popupPanelWidget.canEdit()) {
                let editAnnotationPic = $('<div>');
                editAnnotationPic
                    .attr('id', 'editAnnotationPic')
                    .html('edit')
                    .addClass('popupLeftItem');
                editAnnotationPic.click(function () {
                    window.app.rekall.Rekall('pubSub').publish('image.annotation.edit.open', that);
                });
                imageBoxDiv.append(editAnnotationPic);
            }

            let displayOriginalImage = $('<div>');
            displayOriginalImage
                .attr('id', 'linkToOriginalImage')
                .addClass('popupLeftItem')
                .html('Open original image')
                .click(function () {
                    window.open(that.getDownloadLink(true), '_blank');
                });
            imageBoxDiv.append(displayOriginalImage);
        }
        popupLeftDiv.append(imageBoxDiv);

        if (this.getMetadata('Rekall->Type') != "rekall/marker"
          && this.getMetadata('Rekall->Type') != "rekall/link") {
          let downloadElement = $('<a>');
          downloadElement
            .html("Download File")
            .addClass('popupLeftItem')
            .attr('id', 'popupEditDownload')
            .attr('href', fileDirectLink)
            .attr('download', this.getMetadata('File->File Name'));
          popupLeftDiv.append(downloadElement);
        }

        if (popupPanelWidget.canEdit()) {
            let deleteElement = $('<div>');
            if (this.isMarker()) {
                deleteElement.html("Delete Note");
            } else {
                deleteElement.html("Delete File");
            }
            deleteElement
                .click(function () {
                    window.app.rekall.Rekall('pubSub').publish('alert', {
                        message: "Do you really want to delete this file from the project ?",
                        buttons: "yesnodelete"
                    });
                })
                .addClass('popupLeftItem')
                .attr('id', 'popupEditSupprimer');
            popupLeftDiv.append(deleteElement);
        }
    }

    function createRightPopup(popupPanelWidget) {
        let popupRightTd = $('#popupRight');
        popupRightTd.attr('keyDoc', tag.document.key)
        popupRightTd.empty();

        let commonDataForPanels = {
            canEdit: popupPanelWidget.canEdit(),
            tag: this,
            popupPanelWidget: popupPanelWidget
        };
        let closePopupDiv = $('<div id="closePopupEdit">✕</div>');
        closePopupDiv.click(function(){
            popupPanelWidget.close();
        })
        popupRightTd.append(closePopupDiv);

        let projectNameInput = new AnnotationNamePanelEdition(commonDataForPanels)
        projectNameInput.appendInputEditorDiv(popupRightTd);

        let projectAuthorInput = new AnnotationAuthorPanelEdition(commonDataForPanels);
        projectAuthorInput.appendInputEditorDiv(popupRightTd);

        let tcInput = new AnnotationTCPanelEdition(commonDataForPanels);
        tcInput.appendInputEditorDiv(popupRightTd);

        let commentPanel = new AnnotationCommentPanelEdition(commonDataForPanels);
        commentPanel.appendInputEditorDiv(popupRightTd);

        let linkPanel = new AnnotationLinkPanelEdition(commonDataForPanels);
        linkPanel.appendInputEditorDiv(popupRightTd);

        let speedPanel = new AnnotationSpeedPanelEdition(commonDataForPanels);
        speedPanel.appendInputEditorDiv(popupRightTd);

        let tagPanel = new AnnotationTagPanelEdition(commonDataForPanels);
        tagPanel.appendInputEditorDiv(popupRightTd);
    }

    createRightPopup.call(tag, this);
    createLeftPopup.call(tag, this);
    $('#popupEdit')
        .unbind('click')
        .click(function (event) {
        event.stopPropagation();
        that.ensureLastOpenedEditionClosed();
    });
}
PopupPanelWidget.prototype.show = function() {
    $("#popupSpace").show();
    $("#popupEdit").show();
}
PopupPanelWidget.prototype.close = function() {
    this.ensureLastOpenedEditionClosed();
    $("#popupSpace").hide();
    $("#popupEdit").hide();
    if(this.videoWasPlayingOnContentCreation) {
        window.app.rekall.Rekall().timeline.play();
    }
    window.app.rekall.Rekall('pubSub').publish('popupEdit.Closed', {});
}
