var rekall = new Rekall();
var rekall_common = new Object();
var pubSub = null;
var popupPanel = new PopupPanelWidget({ canEdit: () => rekall_common.owner.canEdit });

window.onload = function() {
	pubSub = window.parent.PubSub;
	window.app.rekall.Rekall('init', {
		PubSub: pubSub
	});
	pubSub.subscribe('mosaic', openMosaic);
	pubSub.subscribe('video', openVideo);
	pubSub.subscribe('open_paste_modal', openPasteModal);
	pubSub.subscribe('close_paste_modal', closePasteModal);
	pubSub.subscribe('updated.project.name', onUpdatedProjectName);
	pubSub.subscribe('alert', openAlertWrapper);
	pubSub.subscribe('roulette.terminated', rouletteEnd);
	pubSub.subscribe('roulette.started', rouletteStart);
	pubSub.subscribe('tag.metadata.updated', onMetaFromDomUpdated);
	pubSub.subscribe('tag.tc.updated', onTcUpdated);
	pubSub.subscribe('image.annotation.edit.open', openImageEdition);

	window.app.rekall.Rekall('init', {
		Rekall: rekall
	});
	window.app.rekall.Rekall().allowProjectDeletion(window.app.Settings.Project.ProjectDeletionEnabled);

	Utils.actionPrefix = "php/";
	Utils.actionSuffix = ".php?r=";

	rekall_common.owner = {"canEdit": false, "author": "", "locationGps": "", "locationName": ""};

	rouletteStart();

	$('#tab_selector_form').change(function (event) {
		pubSub.publish(event.target.value)
	});

	var httpGetData = {"status": 1};
	if(getParameterByName("p") != "") {
		httpGetData.p = getParameterByName("p");
		if(httpGetData.p.length != 40)
			httpGetData.p = (CryptoJS.SHA1(getParameterByName("p").toUpperCase()) + "").toUpperCase();
	}

	$.ajax("php/project.php", {
		type: "POST",
		dataType: "json",
		data: httpGetData,
		success: function(infos) {
			rekall_common = infos;
			if(getParameterByName("w") != "")
				rekall_common.owner.canEdit = false;

			if(rekall_common.owner.canEdit) {
				//Mode preview or not
				$(".editmode").removeClass("editmode");
				$(".empty").show();
				$(".displayMode").hide();

				setEditionControls();

			} else {
				$(".empty").hide();
				$(".displayMode").show();

				$("#projectInfoBtn").click(function(event){
					event.stopPropagation();
					$("#popupSettingsSpace").show();

					$("#popupSettingsTitle").html(window.app.rekall.Rekall('projectName')).removeClass("empty");
					$("#popupSettingsAuthor").html(rekall.project.metadata["Author"]).removeClass("empty");
					$("#popupSettingsEmail").html(rekall.project.metadata["Email"]).removeClass("empty");
					$("#popupSettingsCredits").html(rekall.project.metadata["Comments"].replace(/\n/gi, "<br/>")).removeClass("empty");

				});

				$("#popupSettings").click(function(event){
					event.stopPropagation();
					closeSettingsPopup();
				});

				$("#popupSettingsSpace").click(function(event){
					event.stopPropagation();
					closeSettingsPopup();
				});
			}

			rouletteEnd();
			rekall.loadXMLFile();

			var tmpUrl = shareLink();
			$("#watermarkBox").attr("href",tmpUrl);

		},
		error: function() {
			rouletteEnd();
			rekall.loadXMLFile();
		}
	});

	$("#popupAlertSpace").click(function(event){
		event.stopPropagation();
	});
	$("#popupAlertButtonOk").click(function(event){
		event.stopPropagation();
		$("#popupAlertSpace").hide();
	});

	$("#popupSpace").click(function(event){
		event.stopPropagation();
		popupPanel.close();
	});

	$("#popupAlertButtonCancel").click(function(){
		closeAlert();
	});

	$("#popupAlertButtonYesdelete").click(function(){
		var keyDoc = $("#popupRight").attr("keyDoc");
		deleteFromDom(keyDoc);
	});

	$("#popupAlertButtonYesdeleteproject").click(function(){
		removeProject();
	});

	$("#popupSettingsBtnEmbed").click(function(event){
		event.stopPropagation();
		var tmp = shareEmbed();
		openAlert("textarea",tmp);
	});
	$("#popupSettingsBtnShare").click(function(event){
		event.stopPropagation();
		var tmp = shareLink();
		openAlert("input",tmp);
	});
};


function setEditionControls() {

	//Drag&drop files
	$(document).on({
		dragenter: function (event) {
			event.stopImmediatePropagation();
			event.preventDefault();
		},
		dragleave: function (event) {
			event.stopImmediatePropagation();
			event.preventDefault();
		},
		dragover: function (event) {
			event.stopImmediatePropagation();
			event.preventDefault();
		},
		drop: function (event) {
			if (event.originalEvent.dataTransfer.files.length) {
				event.stopImmediatePropagation();
				event.preventDefault();
				uploadFiles(event.originalEvent.dataTransfer.files);
			}
		}
	});

	$("#flattentimeline").on({
		dragenter: function (event) { /*$(".flattentimeline_item").removeClass("draggable").addClass("drag");*/
		},
		dragleave: function (event) {
			$("#flattentimeline").removeClass("draggable").removeClass("drag");
		},
		dragover: function (event) {
			$("#flattentimeline").removeClass("draggable").addClass("drag");
		},
		drop: function (event) {
			$("#flattentimeline").removeClass("draggable").removeClass("drag");
		}
	});

	$("#left_menu_item_settings").click(function (event) {
		event.stopPropagation();
		rekall.timeline.pause();
		$("#popupSettingsSpace").show();

		if (window.app.rekall.Rekall('projectName') && window.app.rekall.Rekall('projectName') != "") {
			$("#popupSettingsTitle").html(window.app.rekall.Rekall('projectName')).removeClass("empty");
			$("#popupSettingsTitleLabel").show();
		} else {
			$("#popupSettingsTitle").html("+ Add project name").addClass("empty");
			$("#popupSettingsTitleLabel").hide();
		}

		if (rekall.project.metadata["Author"] != "") {
			$("#popupSettingsAuthor").html(rekall.project.metadata["Author"]).removeClass("empty");
			$("#popupSettingsAuthorLabel").show();
		} else {
			$("#popupSettingsAuthor").html("+ Add project author").addClass("empty");
			$("#popupSettingsAuthorLabel").hide();
		}

		if (rekall.project.metadata["Email"] != "") {
			$("#popupSettingsEmail").html(rekall.project.metadata["Email"]).removeClass("empty");
			$("#popupSettingsEmailLabel").show();
		} else {
			$("#popupSettingsEmail").html("+ Add email address").addClass("empty");
			$("#popupSettingsEmailLabel").hide();
		}

		if (rekall.project.metadata["Comments"] != "") {
			$("#popupSettingsCredits").html(rekall.project.metadata["Comments"].replace(/\n/gi, "<br/>")).removeClass("empty");
			$("#popupSettingsCreditsLabel").show();
		} else {
			$("#popupSettingsCredits").html("+ Add project credits").addClass("empty");
			$("#popupSettingsCreditsLabel").hide();
		}

		if (rekall_common.owner.canEdit) {
			$(".empty").show();
		}
	});

	$("#popupSettingsTitleDiv").click(function (event) {
		event.stopPropagation();
		closeSettingsInputs();
		$(this).hide();
		if (!$("#popupSettingsTitle").hasClass("empty")) $("#popupSettingsTitleInput").val($("#popupSettingsTitle").html());
		$("#popupSettingsTitleInput").show().focus();
	});
	$("#popupSettingsAuthorDiv").click(function (event) {
		event.stopPropagation();
		closeSettingsInputs();
		$(this).hide();
		if (!$("#popupSettingsAuthor").hasClass("empty")) $("#popupSettingsAuthorInput").val($("#popupSettingsAuthor").html());
		$("#popupSettingsAuthorInput").show().focus();
	});
	$("#popupSettingsEmailDiv").click(function (event) {
		event.stopPropagation();
		closeSettingsInputs();
		$(this).hide();
		if (!$("#popupSettingsEmail").hasClass("empty")) $("#popupSettingsEmailInput").val($("#popupSettingsEmail").html());
		$("#popupSettingsEmailInput").show().focus();
	});
	$("#popupSettingsCreditsDiv").click(function (event) {
		event.stopPropagation();
		closeSettingsInputs();
		$(this).hide();
		if (!$("#popupSettingsCredits").hasClass("empty")) $("#popupSettingsCreditsInput").val($("#popupSettingsCredits").html());
		$("#popupSettingsCreditsInput").show().focus();
	});

	$(".popupSettingsInput").keyup(function (event) {
		event.stopPropagation();
		if (event.which == 13) {
			closeSettingsInputs();
		}
	});

	$(".popupSettingsInput").click(function (event) {
		event.stopPropagation();
	});

	$("#popupSettingsCreditsInput").unbind("keyup");
	$("#popupSettingsCreditsInput").keyup(function (event) {
		event.stopPropagation();

		var isEnter = false;
		if (event.key !== undefined) {
			if (event.key === 'Enter' && event.altKey) {
			} else if (event.key === 'Enter') isEnter = true;
		} else if (event.keyIdentifier !== undefined) {
			if (event.keyIdentifier === "Enter" && event.altKey) {
			} else if (event.keyIdentifier === 'Enter') isEnter = true;
		} else if (event.keyCode !== undefined) {
			if (event.keyCode === 13 && event.altKey) {
			} else if (event.keyCode === 13) isEnter = true;
		}

		if (isEnter == true) {
			closeSettingsInputs();
		}
	});

	$("#popupSettings").click(function (event) {
		event.stopPropagation();
		closeSettingsInputs();
	});

	$("#popupSettingsSpace").click(function (event) {
		event.stopPropagation();
		closeSettingsPopup();
	});

	$("#left_menu_item_preview").click(function (event) {
		event.stopPropagation();
		window.open(window.app.rekall.Rekall('getUrl', 'projectPreview'), '_blank');
	});

	$("#popupSettingsBtnDownloadXml").click(function (event) {
		event.stopPropagation();
		window.open("php/project.php?downloadXML=1", '_self');
	});
	$("#popupSettingsBtnDelete").click(function (event) {
		event.stopPropagation();
		openAlert("Do you really want to delete this project ?", "yesnodeleteproject");
	});

	$("#btn_add_note").click(function (event) {
		event.stopPropagation();
		rekall.timeline.pause();
		uploadFiles(["New note"]);
	});

	$("#btn_add_file").mousedown(function (event) {
		rekall.timeline.pause();
		$("#left_menu_item_btn_addfile").click();
	});
	$("#left_menu_item_btn_addfile").change(function (event) {
		event.stopPropagation();
		uploadFiles($("#left_menu_item_btn_addfile").get(0).files);
	});

	$("#btn_add_paste").click(function (event) {
		pubSub.publish('open_paste_modal');
	});
	$("#paste_modal button.cancel").click(function (event) {
		pubSub.publish('close_paste_modal');
	});

	$("#paste_modal_content").on("paste", function (event) {
		$('#paste_modal_content').empty();
	});

	$("#paste_modal button.validate").click(function (event) {
		var img = $("#paste_modal img");
		if (img.length < 1)
			return;
		var b64img = img[0].src;
		var data = b64img.match(/data:([a-z]+)\/([a-z]+);base64,(.*)/);
		var type = data[1], subtype = data[2], b64data = data[3];
		var bin_str = atob(b64data);
		var n = b64data.length;
		var uar = new Uint8Array(n);
		while (n--) uar[n] = bin_str.charCodeAt(n);
		var file = new File([uar], 'pasted_' + Math.floor(Math.random() * 100000) + '.' + subtype, {type: type + '/' + subtype});
		$('#left_menu_item_btn_addfile').files += file;
		uploadFiles([file]);
		pubSub.publish('close_paste_modal');
	});

	$("#btn_add_link").click(function (event) {
		event.stopPropagation();
		rekall.timeline.pause();
		$("#popupAddLinkSpace").show();
		$("#popupAddLinkInput").focus();
	});

	$("#popupAddLinkButtonCancel").click(function (event) {
		event.stopPropagation();
		closeAddLinkPopup();
	});
	$("#popupAddLinkSpace").click(function (event) {
		event.stopPropagation();
		closeAddLinkPopup();
	});

	$("#popupAddLinkButtonOk").click(function (event) {
		var myLink = $("#popupAddLinkInput").val();
		addLink(myLink);
	});

	$("#popupAddLinkInput").keyup(function (event) {
		event.stopPropagation();
		if (event.which == 13) {
			var myLink = $("#popupAddLinkInput").val();
			addLink(myLink);
		}
	});

	$("#popupEdit").click(function (event) {
		event.stopPropagation();
		closeInputs();
	});

	$("#popupNom").click(function (event) {
		event.stopPropagation();
		closeInputs();
		$(this).hide();
		$("#popupNomInput").show().focus();
	});

	$("#popupTC").click(function (event) {
		event.stopPropagation();
		closeInputs();
		$("#popupTC").hide();
		$("#popupTCedit").show();
	});

	$(".popupTCeditfield").click(function (event) {
		event.stopPropagation();
	});
	$("#nowTCin").click(function (event) {
		event.stopPropagation();
		var timeCurrent = convertToTime(Math.round(rekall.timeline.timeCurrent));
		$("#popupTCinMin").val(timeCurrent.split(":")[0]);
		$("#popupTCinSec").val(timeCurrent.split(":")[1]);
	});
	$("#nowTCout").click(function (event) {
		event.stopPropagation();
		var timeCurrent = convertToTime(Math.round(rekall.timeline.timeCurrent));
		$("#popupTCoutMin").val(timeCurrent.split(":")[0]);
		$("#popupTCoutSec").val(timeCurrent.split(":")[1]);
	});
	$("#eovTCout").click(function (event) {
		event.stopPropagation();
		var endVideo = rekall.videoPlayer.duration();
		var timeEnd = convertToTime(Math.round(endVideo));
		$("#popupTCoutMin").val(timeEnd.split(":")[0]);
		$("#popupTCoutSec").val(timeEnd.split(":")[1]);
	});

	$("#TCvalidModif").click(function (event) {
		event.stopPropagation();
		var keyDoc = $(this).parent().parent().attr("keydoc");
		var inMin = $("#popupTCinMin").val();
		var inSec = $("#popupTCinSec").val();
		var outMin = $("#popupTCoutMin").val();
		var outSec = $("#popupTCoutSec").val();
		var TCin = (inMin * 60) + (inSec * 1);
		var TCout = (outMin * 60) + (outSec * 1);

		var endVideo = Math.ceil(rekall.videoPlayer.duration());

		if (TCin > TCout) openAlert("Start time must be set before end time", "ok");
		else if (TCout > endVideo) openAlert("End time must not be set after " + convertToTime(endVideo) + " (end of the video)", "ok");
		else {
			setTCFromDom(keyDoc, TCin, TCout);

			$("#popupTCin").html(inMin + ":" + inSec);
			$("#popupTCout").html(outMin + ":" + outSec);

			closeInputs();
		}
	});
	$("#TCinvalidModif").click(function (event) {
		event.stopPropagation();
		var TCin = $("#popupTCin").html().split(":");
		var TCout = $("#popupTCout").html().split(":");
		$("#popupTCinMin").val(TCin[0]);
		$("#popupTCinSec").val(TCin[1]);
		$("#popupTCoutMin").val(TCout[0]);
		$("#popupTCoutSec").val(TCout[1]);
		closeInputs();
	});

	$("#popupLabels").click(function (event) {
		event.stopPropagation();
		closeInputs();
		$(this).hide();
		$("#popupLabelsInput").show().focus();
	});

	$("#popupSpeed").click(function (event) {
		event.stopPropagation();
		closeInputs();
		$(this).hide();
		$("#popupSpeedInput").show().focus();
	});

	$("#popupLegende").click(function (event) {
		event.stopPropagation();
		closeInputs();
		$(this).hide();
		$("#popupLegendeInput").show().focus();
	});

	$("#popupAuthor").click(function (event) {
		event.stopPropagation();
		closeInputs();
		$(this).hide();
		$("#popupAuthorInput").show().focus();
	});

	$("#popupLink").click(function (event) {
		event.stopPropagation();
		closeInputs();
		$(this).hide();
		$("#popupLinkInput").show().focus();
	});

	$(".popupInput").click(function (event) {
		event.stopPropagation();
	});

	$("#popupNomInput").keyup(function (event) {
		event.stopPropagation();
		if (event.which == 13) {
			closeInputs();
		}
	});

	$("#popupAuthorInput").keyup(function (event) {
		event.stopPropagation();
		if (event.which == 13) {
			closeInputs();
		}
	});

	$("#popupLinkInput").keyup(function (event) {
		event.stopPropagation();
		if (event.which == 13) {
			closeInputs();
		}
	});

	$("#popupSetHighlight").click(function (event) {
		event.stopPropagation();
		var keyDoc = $(this).parent().attr("keydoc");
		var isHL = $(this).attr("isHighlight");
		if (isHL == "true") {
			setMetaFromDom(keyDoc, "Rekall->Highlight", "");
			$(this).attr("isHighlight", "false").removeClass("selected");
			$("#popupEdit").removeClass("highlightPopup");
		} else {
			setMetaFromDom(keyDoc, "Rekall->Highlight", "true");
			var tmpColor = $("#popupNom").css("color");
			$(this).attr("isHighlight", "true").addClass("selected");
			$("#popupEdit").addClass("highlightPopup");
		}
	});

	$("#popupEditSupprimer").click(function () {
		openAlert("Do you really want to delete this file from the project ?", "yesnodelete");
	});

	$("#playback_speed_form").change(function (event) {
		console.debug("set playback rate to " + event.target.value);
		window.app.rekall.Rekall().videoPlayer.playbackRate(Number(event.target.value));
	});
}

function closeSettingsPopup() {
	closeSettingsInputs();
	$("#popupSettingsSpace").hide();
}

function addLink(url) {

	if(url.trim()=="") openAlert("Invalid URL");
	else {
		if(url!="") if(url.indexOf("http")!=0) url = "https://"+url;
		uploadFiles(["NewLink"+url]);
		closeAddLinkPopup();
	}
}

function closeAddLinkPopup() {
	$("#popupAddLinkInput").val("");
	$("#popupAddLinkSpace").hide();
}

function openAlert(message, buttons) {
	//Rétro-compatibilité Rekall-Pro
	if((message == undefined) && (buttons == undefined))
		closeAlert();

	if(message=="input"){
		$("#popupAlertMessage").html("Copy this link to share <i>"+ window.app.rekall.Rekall('projectName') +"</i>");
		$(".popupAlertButton").hide();
		$("#popupAlertButtonOk").show();

		$("#popupAlertInput").val(buttons);
		$("#popupAlertInput").show();
		$("#popupAlertTextarea").hide();

		$("#popupAlertSpace").show();
		$("#popupAlertInput").focus().select();

	} else if(message=="textarea"){
		$("#popupAlertMessage").html("Embed <i>"+ window.app.rekall.Rekall('projectName')+"</i> on your webpage");
		$(".popupAlertButton").hide();
		$("#popupAlertButtonOk").show();

		$("#popupAlertTextarea").val(buttons);
		$("#popupAlertInput").hide();
		$("#popupAlertTextarea").show();

		$("#popupAlertSpace").show();
		$("#popupAlertTextarea").focus().select();

	} else {
		$("#popupAlertInput").hide();
		$("#popupAlertTextarea").hide();

		$("#popupAlertMessage").html(message);
		if(buttons == "nobuttons") {
			$(".popupAlertButton").hide();
		}
		else if(buttons == "yesnodelete") {
			$(".popupAlertButton").hide();
			$("#popupAlertButtonYesdelete").show();
			$("#popupAlertButtonCancel").show();
		}
		else if(buttons == "yesnodeleteproject") {
			$(".popupAlertButton").hide();
			$("#popupAlertButtonYesdeleteproject").show();
			$("#popupAlertButtonCancel").show();
		}
		else {
			$(".popupAlertButton").hide();
			$("#popupAlertButtonOk").show();
		}
		$("#popupAlertSpace").show();
	}
}

function openAlertWrapper(message, data){
	openAlert(data.message, data.buttons);
}

function closeAlert() {
	$("#popupAlertMessage").html("");
	$(".popupAlertButton").hide();
	$("#popupAlertSpace").hide();
}

function closeSettingsInputs() {
	$.each($(".popupSettingsInput"), function() {
		if($(this).css("display") != "none") {
			if($(this).attr("id")=="popupSettingsTitleInput") {

				var newName = $(this).val().trim();
				window.app.rekall.Rekall('projectName', newName);
				$(this).val(newName);

				if(newName!="") {
					$("#popupSettingsTitle").html(newName).removeClass("empty");
					$("#popupSettingsTitleLabel").show();
				} else {
					$("#popupSettingsTitle").html("+ Add project name").addClass("empty").show();
					$("#popupSettingsTitleLabel").hide();
				}

			} else if($(this).attr("id")=="popupSettingsAuthorInput") {

				var newAuthor = $(this).val().trim();
				$(this).val(newAuthor);
				setProjectMeta("Author", newAuthor);

				if(newAuthor!="") {
					$("#popupSettingsAuthor").html(newAuthor).removeClass("empty");
					$("#popupSettingsAuthorLabel").show();
				} else {
					$("#popupSettingsAuthor").html("+ Add project author").addClass("empty").show();
					$("#popupSettingsAuthorLabel").hide();
				}

			} else if($(this).attr("id")=="popupSettingsEmailInput") {

				var newEmail = $(this).val().trim();
				$(this).val(newEmail);
				setProjectMeta("Email", newEmail);

				if(newEmail!="") {
					$("#popupSettingsEmail").html(newEmail).removeClass("empty");
					$("#popupSettingsEmailLabel").show();
				} else {
					$("#popupSettingsEmail").html("+ Add email address").addClass("empty").show();
					$("#popupSettingsEmailLabel").hide();
				}

			} else if($(this).attr("id")=="popupSettingsCreditsInput") {

				var newCredits = $(this).val().trim();
				$(this).val(newCredits);
				setProjectMeta("Comments", newCredits);

				if(newCredits!="") {
					$("#popupSettingsCredits").html(newCredits.replace(/\n/gi, "<br/>")).removeClass("empty");
					$("#popupSettingsCreditsLabel").show();
				} else {
					$("#popupSettingsCredits").html("+ Add project credits").addClass("empty").show();
					$("#popupSettingsCreditsLabel").hide();
				}

			}
		}

	});
	$(".popupSettingsInput").hide();
	$(".popupSettingsTxt").show();
}

function closeInputs() {
	popupPanel.ensureLastOpenedEditionClosed();
	$(".popupRightItem").show();
}

function closeEdit() {
	popupPanel.close();
	window.app.rekall.Rekall('pubSub').publish('popupEdit.Closed', {});
}

function openVideo() {
	window.app.rekall.Rekall('mosaicWidget').hide();
	$('#video_tab').show();
	$('#left_menu_controls').show();
}

function openMosaic() {
	$('#video_tab').hide();
	if(rekall_common.owner.canEdit) {
		$('#left_menu_controls').hide();
	}
    window.app.rekall.Rekall('mosaicWidget').show();
}

function fillPopupEdit(tag) {

	popupPanel.createPopupContent(tag);

	if(rekall_common.owner.canEdit) {
		$(".empty").show();
		$(".displayMode").hide();
	} else {
		$(".empty").hide();
		$(".editmode").hide();
		$(".displayMode").show();
	}

	popupPanel.show();
}

function convertToTime(seconds) {
	var minutes = Math.floor(seconds/60);
	seconds = Math.floor(seconds-(minutes*60));
	if(minutes<10) minutes="0"+minutes;
	if(seconds<10) seconds="0"+seconds;
	var time = minutes+":"+seconds;
	return time;
}

function removeProject() {
	rouletteStart();
	$.ajax("php/project.php", {
		type: "POST",
		dataType: "json",
		data: {"remove": 1},
		success: function(retour) {
			rouletteEnd();
			window.app.rekall.Rekall('openUrl', 'home');
		},
		error: function() {
			openAlert("Server error. Try again.");
			rouletteEnd();
		}
	});
}

function onUpdatedProjectName(msg, data) {
	setProjectMeta('Title', data.projectName);
}

function setProjectMeta(metaType, meta) {
	rouletteStart();
	$.ajax("php/project.php", {
		type: "POST",
		dataType: "json",
		data: {"metadataKey": metaType, "metadataValue": meta.replace(/'/g, '’')},
		success: function(retour) {
			rekall.project.metadata[metaType] = meta;
			rouletteEnd();
		},
		error: function() {
			openAlert("Server error. Try again.");
			rouletteEnd();
		}
	});
}

function setMetaFromDom(keyDoc, metaType, meta) {
	rouletteStart();
	$.ajax("php/project.php", {
		type: "POST",
		dataType: "json",
		data: {"key": keyDoc, "metadataKey": metaType, "metadataValue": meta.indexOf('\'') == -1 ? meta : meta.replace(/'/g, '’')},
		success: function(retour) {
			rouletteEnd();
			rekall.loadXMLFile();
		},
		error: function() {
			openAlert("Server error. Try again.");
			rouletteEnd();
		}
	});
}

function onMetaFromDomUpdated(message, data){
	setMetaFromDom(data.tag.document.key, data.metadata, data.tag.getMetadata(data.metadata));
}

function onTcUpdated(message, data){
	setTCFromDom(data.tag.document.key, data.tag.getTimeStart(), data.tag.getTimeEnd());
}

function setTCFromDom(keyDoc, TCin, TCout) {
	rouletteStart();
	$.ajax("php/project.php", {
		type: "POST",
		dataType: "json",
		data: {"key": keyDoc, "tcIn": TCin, "tcOut": TCout},
		success: function(retour) {
			rouletteEnd();
			rekall.loadXMLFile();
		},
		error: function() {
			alert("Error");
			openAlert("Server error. Try again.");
			rouletteEnd();
		}
	});
}

function deleteFromDom(keyDoc) {
	rouletteStart();
	$.ajax("php/project.php", {
		type: "POST",
		dataType: "json",
		data: {"key": keyDoc, "remove": 1},
		success: function(retour) {
			deleteFromDomFinished();
			rouletteEnd();
			rekall.loadXMLFile();
		},
		error: function() {
			openAlert("Server error. Try again.");
			rouletteEnd();
		}
	});
}

function deleteFromDomFinished() {
	closeAlert();
	popupPanel.close();
}

//Gestion d'upload
var filesToUpload = [], fileIsUploading = false;
function uploadFiles(files, additionnal_post_data={}, callBack = undefined) {
	$.each(files, function(index, file) {
		var formData = new FormData();

		if(file.name != undefined) {
			var fileDateTime = moment(file.lastModifiedDate);

			if (file.size > rekall_common.uploadMax) {
				openAlert("File size is too large! Maximum is " + rekall_common.uploadMax + " bytes.");
				return;
			}

			//Données du formulaire
			if($("uploadForm")[0] != undefined)
				formData = new FormData($('form')[0]); //à vérifier
			else {
				formData.append("fileToUpload", file);
				formData.append("date", fileDateTime.format("YYYY:MM:DD HH:mm:ss"));
			}
		}
		else if(file.indexOf("NewLink") == 0){
			formData.append("name", "NewLink");
			formData.append("link", file.replace("NewLink",""));
			formData.append("type", "rekall/link");
		}
		else {
			formData.append("name", file);
		}

		var tcOut = rekall.timeline.timeCurrent + 15;
		if(tcOut>=rekall.videoPlayer.duration()) tcOut = rekall.videoPlayer.duration();

		formData.append("tcIn", 		rekall.timeline.timeCurrent);
		formData.append("tcOut", 		tcOut);
		formData.append("author",       rekall_common.owner.author);
		formData.append("locationGps",  rekall_common.owner.locationGps);
		formData.append("locationName", rekall_common.owner.locationName);
        Object.keys(additionnal_post_data).forEach(k => formData.append(k, additionnal_post_data[k]));

		if(formData != undefined) {
			filesToUpload.push({
				file: 		 file,
				url:  		 'php/upload.php',
				type: 		 'POST',
				data:		 formData,
				cache:       false,
				contentType: false,
				processData: false,
				xhr: function() {
					var myXhr = $.ajaxSettings.xhr();
					if(myXhr.upload) {
						myXhr.upload.addEventListener('progress', function(event) {
							rouletteProgress(Math.floor(event.loaded / event.total * 100));
						}, false);
					}
					return myXhr;
				},
				success:    function(data) {
					try {
						console.log(data);
						data = JSON.parse(data);
						Tag.keyToOpenAfterLoading = data.files[0].key;
						if(data.files[0].code <= 0)
							openAlert("Uploading error. " + data.files[0].error);
					}
					catch(err) {
						openAlert("Uploading error. Try again.");
					}
					window.document.title = "Rekall";
					fileIsUploading = false;

					rouletteEnd();
					uploadFilesNext();
					if(callBack && typeof callBack === 'function'){
						callBack();
					}
				},
				error: function(data) {
					openAlert("Uploading error. Try again.");
					window.document.title = "Rekall";
					fileIsUploading = false;

					rouletteEnd();
					uploadFilesNext();
					if(callBack && typeof callBack === 'function'){
						callBack();
					}
				}
			});
			uploadFilesNext();
		}
	});
}

function uploadFilesNext() {
	if(!fileIsUploading) {
		if(filesToUpload.length > 0) {
			fileIsUploading = true;
			rouletteStart(true);
			$.ajax(filesToUpload[0]);

			if(filesToUpload[0].file.name != undefined)
				openAlert("Starting upload of " + filesToUpload[0].file.name);
			else
				openAlert("Creation of " + filesToUpload[0].file + " in progress");

			filesToUpload.splice(0, 1);
			uploadFilesNext();
		}
		else {
			rekall.loadXMLFile();
		}
	}
}

//Code d'embarquement du projet Rekall
function shareEmbed() {
	var width = 960, height = round(width * 0.44);
	var embedUrl = '<iframe src="' + window.app.rekall.Rekall('getUrl', 'projectPreview') + '" width="' + width + '" height="' + height + '" frameborder="0" webkitallowfullscreen mozallowfullscreen allowfullscreen></iframe>';
	embedUrl += '<p><a href="' + window.app.rekall.Rekall('getUrl', 'projectPreview') + '">' + window.app.rekall.Rekall('projectName') + '</a> from '+rekall.project.metadata["Author"]+' on <a href="http://www.memorekall.fr">MemoRekall</a>.</p>';
	console.log(embedUrl);
	return embedUrl;
}

function shareLink() {
	var width = 960, height = round(width * 0.44);
	var embedUrl = window.app.rekall.Rekall('getUrl', 'projectPreview');
	return embedUrl;
}

function rouletteStart(isProgress) {
	if(isProgress == undefined)
		console.log("Début de la roulette infinie");
	else
		console.log("Début de la roulette avec progression");
}

function rouletteProgress(progress) {
	console.log("Progression de la roulette @ " + progress + "%");
	window.document.title = "Téléchargement " + progress + "%";
}

function rouletteEnd() {
	console.log("Fin de la roulette");
	closeAlert();
}

function getParameterByName(name) {
    name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
    var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"),
        results = regex.exec(location.search);
    return results === null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
}

$(window).resize(function(e) {
	if(rekall.videoPlayer != undefined) {
		rekall.videoPlayer.width (($("#container").width() - ($("#left_menu").width()+3) - ($("#flattentimeline").width()+6)) + "px");
		rekall.videoPlayer.height(($("#container").height()) + "px");
		$("#video").addClass("resized");
	}
});
$(window).trigger("resize");

function openPasteModal() {
  $('#paste_modal').css('display', 'flex');
  $('#paste_modal_content').first().focus();
  rekall.timeline.pause();
}

function closePasteModal() {
  $('#paste_modal').hide();
  $('#paste_modal_content').empty();
}

function openImageEdition(message, tag) {
	$('#edit_pic_modal').show();
	let keyDoc = tag.document.key;
	let annotationEdit = $('#annotation_img_edit')[0];
	$('#annotation_img_edit').attr('src', tag.getDownloadLink(true));

	function loadImage(url, elem) {
		return new Promise((resolve, reject) => {
			elem.onload = () => resolve(elem);
			elem.onerror = reject;
			elem.src = url;
		});
	}
	function openMakerArea() {
		// eslint-disable-next-line no-empty
		let markerArea = new markerjs2.MarkerArea($('#annotation_img_edit')[0]);
		let imageState;

		if (tag.getMetadata("Rekall->MarkerjsState") && tag.getMetadata("Rekall->MarkerjsState").trim().length) {
			imageState = JSON.parse(atob(tag.getMetadata("Rekall->MarkerjsState")));
		}

		markerArea.targetRoot = document.getElementById('edit_pic_modal');
		markerArea.addEventListener('render', (event) => {
			imageState = markerArea.getState();
			markerArea.close(true);
			tag.setMetadata("Rekall->MarkerjsState", btoa(JSON.stringify(imageState)));

			let b64img = event.dataUrl;
			let data = b64img.match(/data:([a-z]+)\/([a-z]+);base64,(.*)/);
			let type = data[1], subtype = data[2], b64data = data[3];
			let bin_str = atob(b64data);
			let n = b64data.length;
			let uar = new Uint8Array(n);
			while (n--) uar[n] = bin_str.charCodeAt(n);
			let file = new File([uar], keyDoc.substr(1), {type: type + '/' + subtype});
			$('#left_menu_item_btn_addfile').files = [file];
			uploadFiles([file], {'edited': 1}, function () {
				$('#popupImg').attr('src', tag.getDownloadLink());
			});
			setMetaFromDom(tag.document.key, "Rekall->MarkerjsState", tag.getMetadata('Rekall->MarkerjsState'));
		});

		markerArea.addEventListener('close', () => {
			$('#edit_pic_modal').hide();
		});
		markerArea.renderAtNaturalSize = true;
		markerArea.show();
		if (imageState)
			markerArea.restoreState(imageState);
	}

	async function changeImageSrc() {
		await loadImage(tag.getDownloadLink(true), annotationEdit);
		openMakerArea();
	}

	// noinspection JSIgnoredPromiseFromCall
	changeImageSrc();
}
