<?php

namespace App\Tests\functional;

use Symfony\Bundle\FrameworkBundle\KernelBrowser;
use Symfony\Bundle\FrameworkBundle\Test\WebTestCase;
use Symfony\Component\DomCrawler\Form;

class LoginTest extends WebTestCase
{
    private KernelBrowser $client;
    private Form $form;

    protected function setUp(): void
    {
        self::ensureKernelShutdown();
        $this->client = static::createClient();
        $crawler = $this->client->request('GET', '/login');
        $this->assertResponseIsSuccessful();

        $this->client->enableProfiler();
        $submit_button = $crawler->selectButton('Log in');
        $this->form = $submit_button->form();
    }

    public function testRegisteredUserWithEnabledAccountIsRedirectedToCapsulesPageWhenSubmittingLoginForm(): void
    {
        $this->form['email'] = 'defaultUser@localhost.com';
        $this->form['password'] = "password";

        $this->client->submit($this->form);

        $this->assertResponseRedirects(
            '/en/my_capsules',
            302,
            'Once the user is logged in, he should be redirected to its capsules lists'
        );

        $this->client->followRedirect();

        $this->assertResponseIsSuccessful('/en/my_capsules');
    }

    public function testRegisteredUserWithDisabledAccountIsRedirectedToLoginPageWhenSubmittingLoginForm(): void
    {
        $this->form['email'] = 'notVerified@localhost.com';
        $this->form['password'] = "password";

        $this->client->submit($this->form);

        $this->assertResponseRedirects(
            '/login',
            302,
            '
            A registered user with a disabled account is redirected to login page when trying to log in'
        );

        $this->client->followRedirect();

        $this->assertResponseIsSuccessful('/login');
    }

    public function testNonRegisteredUserIsRedirectedToLoginPageWhenSubmittingLoginForm(): void
    {
        $this->form['email'] = 'notRegistered@localhost.com';
        $this->form['password'] = "password";

        $this->client->submit($this->form);

        $this->assertResponseRedirects(
            '/login',
            302,
            'A non registered user is redirected to login page when trying to log in'
        );

        $this->client->followRedirect();

        $this->assertResponseIsSuccessful('/login');
    }
}
